@@ -1,8 +1,8 @@
 package t::800_utils::Tracker::HTTP;
 use Net::BitTorrent::Protocol qw[:bencode :compact];    # IPv6
-use Any::Moose;
+use Moo;
 use AnyEvent::Socket;
-
+use Test::More;
 #
 extends 't::800_utils::Tracker';
 
@@ -22,7 +22,7 @@ sub _build_socket {
                 on_read => sub {
                     $s->on_read($hdl, $fh, $paddr, $host, $port, @_);
                 },
-                on_eof => sub { warn 'bye!' }
+                on_eof => sub { note 'bye!' }
             );
         },
         sub {
@@ -84,13 +84,13 @@ sub on_read {
                             ip      => $_->{'address'}->[0],
                             port    => $_->{'address'}->[1]
                            }
-                           } @peers
+                       } @peers
                     )
                 )
             };
         }
-        elsif ($path eq '/scrape.pl') { warn 'Scrape!' }
-        else                          { warn 'NFI!' }
+        elsif ($path eq '/scrape.pl') { note 'Scrape!' }
+        else                          { note 'NFI!' }
     }
     $h->rbuf = '';
     $body = bencode $body if ref $body;
@@ -116,7 +116,7 @@ CPAN ID: SANKO
 
 =head1 License and Legal
 
-Copyright (C) 2008-2012 by Sanko Robinson <sanko@cpan.org>
+Copyright (C) 2008-2013 by Sanko Robinson <sanko@cpan.org>
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of
